@@ -1,12 +1,7 @@
-use google_cloud_googleapis::{Code, Status};
-
-use google_cloud_gax::invoke::{invoke_reuse, TryAs};
-use google_cloud_googleapis::spanner::v1::{
-    commit_request, transaction_options, Mutation, TransactionOptions,
-};
+use google_cloud_gax::retry::{invoke_fn, TryAs};
+use google_cloud_googleapis::spanner::v1::{commit_request, transaction_options, Mutation, TransactionOptions};
 
 use crate::apiv1::conn_pool::ConnectionManager;
-use crate::retry::{new_default_tx_retry, new_tx_retry_with_codes};
 use crate::session::{ManagedSession, SessionConfig, SessionError, SessionManager};
 use crate::statement::Statement;
 use crate::transaction::{CallOptions, QueryOptions};
@@ -14,6 +9,9 @@ use crate::transaction_ro::{BatchReadOnlyTransaction, ReadOnlyTransaction};
 use crate::transaction_rw::{commit, CommitOptions, ReadWriteTransaction};
 use crate::value::{Timestamp, TimestampBound};
 
+use crate::retry::TransactionRetrySetting;
+use google_cloud_gax::cancel::CancellationToken;
+use google_cloud_gax::grpc::{Code, Status};
 use std::future::Future;
 use std::pin::Pin;
 use std::sync::Arc;
@@ -100,7 +98,7 @@ pub enum InitializationError {
     FailedToCreateSessionPool(#[from] Status),
 
     #[error(transparent)]
-    FailedToCreateChannelPool(#[from] google_cloud_grpc::conn::Error),
+    FailedToCreateChannelPool(#[from] google_cloud_gax::conn::Error),
 
     #[error("invalid config: {0}")]
     InvalidConfig(String),
@@ -188,8 +186,7 @@ impl Client {
             Err(_) => None,
         };
         let conn_pool = ConnectionManager::new(pool_size, emulator_host).await?;
-        let session_manager =
-            SessionManager::new(database, conn_pool, config.session_config).await?;
+        let session_manager = SessionManager::new(database, conn_pool, config.session_config).await?;
         session_manager.schedule_refresh();
 
         Ok(Client {
@@ -206,18 +203,13 @@ impl Client {
     /// where only a single read or query is needed.  This is more efficient than
     /// using read_only_transaction for a single read or query.
     pub async fn single(&self) -> Result<ReadOnlyTransaction, TxError> {
-        return self
-            .single_with_timestamp_bound(TimestampBound::strong_read())
-            .await;
+        return self.single_with_timestamp_bound(TimestampBound::strong_read()).await;
     }
 
     /// single provides a read-only snapshot transaction optimized for the case
     /// where only a single read or query is needed.  This is more efficient than
     /// using read_only_transaction for a single read or query.
-    pub async fn single_with_timestamp_bound(
-        &self,
-        tb: TimestampBound,
-    ) -> Result<ReadOnlyTransaction, TxError> {
+    pub async fn single_with_timestamp_bound(&self, tb: TimestampBound) -> Result<ReadOnlyTransaction, TxError> {
         let session = self.get_session().await?;
         let result = ReadOnlyTransaction::single(session, tb).await?;
         Ok(result)
@@ -238,9 +230,7 @@ impl Client {
         options: ReadOnlyTransactionOption,
     ) -> Result<ReadOnlyTransaction, TxError> {
         let session = self.get_session().await?;
-        let result =
-            ReadOnlyTransaction::begin(session, options.timestamp_bound, options.call_options)
-                .await?;
+        let result = ReadOnlyTransaction::begin(session, options.timestamp_bound, options.call_options).await?;
         Ok(result)
     }
 
@@ -263,9 +253,7 @@ impl Client {
         options: ReadOnlyTransactionOption,
     ) -> Result<BatchReadOnlyTransaction, TxError> {
         let session = self.get_session().await?;
-        let result =
-            BatchReadOnlyTransaction::begin(session, options.timestamp_bound, options.call_options)
-                .await?;
+        let result = BatchReadOnlyTransaction::begin(session, options.timestamp_bound, options.call_options).await?;
         Ok(result)
     }
 
@@ -296,21 +284,21 @@ impl Client {
         stmt: Statement,
         options: PartitionedUpdateOption,
     ) -> Result<i64, TxError> {
-        let mut ro = new_tx_retry_with_codes(vec![Code::Aborted, Code::Internal]);
+        let ro = TransactionRetrySetting::new(vec![Code::Aborted, Code::Internal]);
         let session = Some(self.get_session().await?);
 
         // reuse session
-        return invoke_reuse(
+        return invoke_fn(
+            options.begin_options.cancel.clone(),
+            Some(ro),
             |session| async {
-                let mut tx = match ReadWriteTransaction::begin_partitioned_dml(
-                    session.unwrap(),
-                    options.begin_options.clone(),
-                )
-                .await
-                {
-                    Ok(tx) => tx,
-                    Err(e) => return Err((TxError::GRPC(e.status), Some(e.session))),
-                };
+                let mut tx =
+                    match ReadWriteTransaction::begin_partitioned_dml(session.unwrap(), options.begin_options.clone())
+                        .await
+                    {
+                        Ok(tx) => tx,
+                        Err(e) => return Err((TxError::GRPC(e.status), Some(e.session))),
+                    };
                 let qo = match options.query_options.clone() {
                     Some(o) => o,
                     None => QueryOptions::default(),
@@ -320,7 +308,6 @@ impl Client {
                     .map_err(|e| (TxError::GRPC(e), tx.take_session()))
             },
             session,
-            &mut ro,
         )
         .await;
     }
@@ -334,13 +321,8 @@ impl Client {
     /// apply's default replay protection may require an additional RPC.  So this
     /// method may be appropriate for latency sensitive and/or high throughput blind
     /// writing.
-    pub async fn apply_at_least_once(
-        &self,
-        ms: Vec<Mutation>,
-    ) -> Result<Option<Timestamp>, TxError> {
-        return self
-            .apply_at_least_once_with_option(ms, CommitOptions::default())
-            .await;
+    pub async fn apply_at_least_once(&self, ms: Vec<Mutation>) -> Result<Option<Timestamp>, TxError> {
+        return self.apply_at_least_once_with_option(ms, CommitOptions::default()).await;
     }
 
     /// apply_at_least_once may attempt to apply mutations more than once; if
@@ -357,15 +339,15 @@ impl Client {
         ms: Vec<Mutation>,
         options: CommitOptions,
     ) -> Result<Option<Timestamp>, TxError> {
-        let mut ro = new_default_tx_retry();
+        let ro = TransactionRetrySetting::default();
         let mut session = self.get_session().await?;
 
-        return invoke_reuse(
+        return invoke_fn(
+            options.call_options.cancel.clone(),
+            Some(ro),
             |session| async {
                 let tx = commit_request::Transaction::SingleUseTransaction(TransactionOptions {
-                    mode: Some(transaction_options::Mode::ReadWrite(
-                        transaction_options::ReadWrite {},
-                    )),
+                    mode: Some(transaction_options::Mode::ReadWrite(transaction_options::ReadWrite {})),
                 });
                 match commit(session, ms.clone(), tx, options.clone()).await {
                     Ok(s) => Ok(match s.commit_timestamp {
@@ -376,16 +358,13 @@ impl Client {
                 }
             },
             &mut session,
-            &mut ro,
         )
         .await;
     }
 
     /// Apply applies a list of mutations atomically to the database.
     pub async fn apply(&self, ms: Vec<Mutation>) -> Result<Option<Timestamp>, TxError> {
-        return self
-            .apply_with_option(ms, ReadWriteTransactionOption::default())
-            .await;
+        return self.apply_with_option(ms, ReadWriteTransactionOption::default()).await;
     }
 
     pub async fn apply_with_option(
@@ -395,7 +374,7 @@ impl Client {
     ) -> Result<Option<Timestamp>, TxError> {
         let result: Result<(Option<Timestamp>, ()), TxError> = self
             .read_write_transaction_sync_with_option(
-                |tx| {
+                |tx, _cancel| {
                     tx.buffer_write(ms.to_vec());
                     Ok(())
                 },
@@ -423,14 +402,12 @@ impl Client {
     ///
     /// See <https://godoc.org/cloud.google.com/go/spanner#ReadWriteTransaction> for
     /// more details.
-    pub async fn read_write_transaction<'a, T, E, F>(
-        &self,
-        f: F,
-    ) -> Result<(Option<Timestamp>, T), E>
+    pub async fn read_write_transaction<'a, T, E, F>(&self, f: F) -> Result<(Option<Timestamp>, T), E>
     where
         E: TryAs<Status> + From<SessionError> + From<Status>,
         F: for<'tx> Fn(
             &'tx mut ReadWriteTransaction,
+            Option<CancellationToken>,
         ) -> Pin<Box<dyn Future<Output = Result<T, E>> + Send + 'tx>>,
     {
         return self
@@ -465,24 +442,25 @@ impl Client {
         E: TryAs<Status> + From<SessionError> + From<Status>,
         F: for<'tx> Fn(
             &'tx mut ReadWriteTransaction,
+            Option<CancellationToken>,
         ) -> Pin<Box<dyn Future<Output = Result<T, E>> + Send + 'tx>>,
     {
         let (bo, co) = Client::split_read_write_transaction_option(options);
 
-        let mut ro = new_default_tx_retry();
+        let ro = TransactionRetrySetting::default();
         let session = Some(self.get_session().await?);
-
+        let cancel = bo.cancel.clone();
         // must reuse session
-        return invoke_reuse(
+        return invoke_fn(
+            cancel.clone(),
+            Some(ro),
             |session| async {
-                let mut tx = self
-                    .create_read_write_transaction::<E>(session, bo.clone())
-                    .await?;
-                let result = f(&mut tx).await;
+                let cancel = cancel.clone().map(|v| v.child_token());
+                let mut tx = self.create_read_write_transaction::<E>(session, bo.clone()).await?;
+                let result = f(&mut tx, cancel).await;
                 tx.finish(result, Some(co.clone())).await
             },
             session,
-            &mut ro,
         )
         .await;
     }
@@ -494,7 +472,7 @@ impl Client {
 
     async fn read_write_transaction_sync_with_option<T, E>(
         &self,
-        f: impl Fn(&mut ReadWriteTransaction) -> Result<T, E>,
+        f: impl Fn(&mut ReadWriteTransaction, Option<CancellationToken>) -> Result<T, E>,
         options: ReadWriteTransactionOption,
     ) -> Result<(Option<Timestamp>, T), E>
     where
@@ -502,20 +480,21 @@ impl Client {
     {
         let (bo, co) = Client::split_read_write_transaction_option(options);
 
-        let mut ro = new_default_tx_retry();
+        let ro = TransactionRetrySetting::default();
         let session = Some(self.get_session().await?);
 
         // reuse session
-        return invoke_reuse(
+        let cancel = bo.cancel.clone();
+        return invoke_fn(
+            cancel.clone(),
+            Some(ro),
             |session| async {
-                let mut tx = self
-                    .create_read_write_transaction::<E>(session, bo.clone())
-                    .await?;
-                let result = f(&mut tx);
+                let cancel = cancel.clone().map(|v| v.child_token());
+                let mut tx = self.create_read_write_transaction::<E>(session, bo.clone()).await?;
+                let result = f(&mut tx, cancel);
                 tx.finish(result, Some(co.clone())).await
             },
             session,
-            &mut ro,
         )
         .await;
     }
@@ -537,9 +516,7 @@ impl Client {
         return self.sessions.get().await;
     }
 
-    fn split_read_write_transaction_option(
-        options: ReadWriteTransactionOption,
-    ) -> (CallOptions, CommitOptions) {
+    fn split_read_write_transaction_option(options: ReadWriteTransactionOption) -> (CallOptions, CommitOptions) {
         (options.begin_options, options.commit_options)
     }
 }
