@@ -1,14 +1,15 @@
 use std::ops::DerefMut;
 use std::sync::atomic::AtomicI64;
 
-use google_cloud_googleapis::Status;
+use google_cloud_gax::cancel::CancellationToken;
+use google_cloud_gax::retry::RetrySetting;
 use prost_types::Struct;
 
-use google_cloud_gax::call_option::BackoffRetrySettings;
+use google_cloud_gax::grpc::Status;
 use google_cloud_googleapis::spanner::v1::request_options::Priority;
 use google_cloud_googleapis::spanner::v1::{
-    execute_sql_request::QueryMode, execute_sql_request::QueryOptions as ExecuteQueryOptions,
-    ExecuteSqlRequest, ReadRequest, RequestOptions, TransactionSelector,
+    execute_sql_request::QueryMode, execute_sql_request::QueryOptions as ExecuteQueryOptions, ExecuteSqlRequest,
+    ReadRequest, RequestOptions, TransactionSelector,
 };
 
 use crate::key::{Key, KeySet};
@@ -21,14 +22,16 @@ use crate::statement::Statement;
 pub struct CallOptions {
     /// Priority is the RPC priority to use for the read operation.
     pub priority: Option<Priority>,
-    pub call_setting: Option<BackoffRetrySettings>,
+    pub retry: Option<RetrySetting>,
+    pub cancel: Option<CancellationToken>,
 }
 
 impl Default for CallOptions {
     fn default() -> Self {
         CallOptions {
             priority: None,
-            call_setting: None,
+            retry: None,
+            cancel: None,
         }
     }
 }
@@ -94,9 +97,7 @@ impl Transaction {
     ///
     /// query returns only row data, without a query plan or execution statistics.
     pub async fn query(&mut self, statement: Statement) -> Result<RowIterator<'_>, Status> {
-        return self
-            .query_with_option(statement, QueryOptions::default())
-            .await;
+        return self.query_with_option(statement, QueryOptions::default()).await;
     }
 
     /// query executes a query against the database. It returns a RowIterator for
@@ -124,14 +125,8 @@ impl Transaction {
             request_options: Transaction::create_request_options(options.call_options.priority),
         };
         let session = self.session.as_mut().unwrap().deref_mut();
-        return RowIterator::new(
-            session,
-            Box::new(StatementReader {
-                request,
-                call_setting: options.call_options.call_setting,
-            }),
-        )
-        .await;
+        let reader = Box::new(StatementReader { request });
+        RowIterator::new(session, reader, Some(options.call_options)).await
     }
 
     /// read returns a RowIterator for reading multiple rows from the database.
@@ -141,9 +136,8 @@ impl Transaction {
         columns: &[&str],
         key_set: impl Into<KeySet>,
     ) -> Result<RowIterator<'_>, Status> {
-        return self
-            .read_with_option(table, columns, key_set, ReadOptions::default())
-            .await;
+        self.read_with_option(table, columns, key_set, ReadOptions::default())
+            .await
     }
 
     /// read returns a RowIterator for reading multiple rows from the database.
@@ -168,23 +162,12 @@ impl Transaction {
         };
 
         let session = self.as_mut_session();
-        return RowIterator::new(
-            session,
-            Box::new(TableReader {
-                request,
-                call_setting: options.call_options.call_setting,
-            }),
-        )
-        .await;
+        let reader = Box::new(TableReader { request });
+        RowIterator::new(session, reader, Some(options.call_options)).await
     }
 
     /// read returns a RowIterator for reading multiple rows from the database.
-    pub async fn read_row(
-        &mut self,
-        table: &str,
-        columns: &[&str],
-        key: Key,
-    ) -> Result<Option<Row>, Status> {
+    pub async fn read_row(&mut self, table: &str, columns: &[&str], key: Key) -> Result<Option<Row>, Status> {
         return self
             .read_row_with_option(table, columns, key, ReadOptions::default())
             .await;
@@ -198,10 +181,11 @@ impl Transaction {
         key: Key,
         options: ReadOptions,
     ) -> Result<Option<Row>, Status> {
+        let call_options = options.call_options.clone();
         let mut reader = self
             .read_with_option(table, columns, KeySet::from(key), options)
             .await?;
-        return reader.next().await;
+        return reader.next(Some(call_options)).await;
     }
 
     pub(crate) fn get_session_name(&self) -> String {
