@@ -10,15 +10,14 @@ use serial_test::serial;
 
 mod common;
 use common::*;
+use google_cloud_gax::grpc::Status;
 use google_cloud_spanner::reader::{AsyncIterator, RowIterator};
 use google_cloud_spanner::transaction_rw::ReadWriteTransaction;
 
-use google_cloud_googleapis::Status;
-
 pub async fn all_rows(mut itr: RowIterator<'_>) -> Result<Vec<Row>, Status> {
     let mut rows = vec![];
     loop {
-        match itr.next().await {
+        match itr.next(None).await {
             Ok(row) => {
                 if row.is_some() {
                     rows.push(row.unwrap());
@@ -59,8 +58,8 @@ async fn test_mutation_and_statement() {
         stmt1.add_param("UserId", &past_user);
         let mut stmt2 = Statement::new("INSERT INTO UserItem (UserId,ItemId,Quantity,UpdatedAt) VALUES(@UserId,10,1000,PENDING_COMMIT_TIMESTAMP())");
         stmt2.add_param("UserId", &past_user);
-        tx.update(stmt1).await?;
-        return tx.update(stmt2).await;
+        tx.update( stmt1).await?;
+        return tx.update( stmt2).await;
     }.await;
 
     let result = tx.finish(result, None).await;
@@ -91,15 +90,12 @@ async fn test_partitioned_dml() {
         .await
         .unwrap();
 
-    let mut tx =
-        match ReadWriteTransaction::begin_partitioned_dml(session, CallOptions::default()).await {
-            Ok(tx) => tx,
-            Err(e) => panic!("begin first error {:?}", e.status),
-        };
+    let mut tx = match ReadWriteTransaction::begin_partitioned_dml(session, CallOptions::default()).await {
+        Ok(tx) => tx,
+        Err(e) => panic!("begin first error {:?}", e.status),
+    };
     let result = async {
-        let stmt1 = Statement::new(
-            "UPDATE User SET NullableString = 'aaa' WHERE NullableString IS NOT NULL",
-        );
+        let stmt1 = Statement::new("UPDATE User SET NullableString = 'aaa' WHERE NullableString IS NOT NULL");
         tx.update(stmt1).await
     }
     .await;
@@ -109,10 +105,7 @@ async fn test_partitioned_dml() {
 
     let session = create_session().await;
     let mut tx = read_only_transaction(session).await;
-    let reader = tx
-        .read("User", &["NullableString"], Key::key(&user_id))
-        .await
-        .unwrap();
+    let reader = tx.read("User", &["NullableString"], Key::key(&user_id)).await.unwrap();
     let row = all_rows(reader).await.unwrap().pop().unwrap();
     let value = row.column_by_name::<String>("NullableString").unwrap();
     assert_eq!(value, "aaa");
@@ -134,8 +127,7 @@ async fn test_rollback() {
         Err(e) => panic!("begin first error {:?}", e.status),
     };
     let result = async {
-        let mut stmt1 =
-            Statement::new("UPDATE User SET NullableString = 'aaaaaaa' WHERE UserId = @UserId");
+        let mut stmt1 = Statement::new("UPDATE User SET NullableString = 'aaaaaaa' WHERE UserId = @UserId");
         stmt1.add_param("UserId", &past_user);
         tx.update(stmt1).await?;
 
@@ -147,10 +139,7 @@ async fn test_rollback() {
     let _ = tx.finish(result, None).await;
     let session = create_session().await;
     let mut tx = read_only_transaction(session).await;
-    let reader = tx
-        .read("User", &user_columns(), Key::key(&past_user))
-        .await
-        .unwrap();
+    let reader = tx.read("User", &user_columns(), Key::key(&past_user)).await.unwrap();
     let row = all_rows(reader).await.unwrap().pop().unwrap();
     let ts = cr.commit_timestamp.as_ref().unwrap();
     let ts = Utc.timestamp(ts.seconds, ts.nanos as u32);
@@ -204,9 +193,7 @@ async fn assert_data(
     );
     assert!(user_items.is_empty());
 
-    let mut user_characters = row
-        .column_by_name::<Vec<UserCharacter>>("UserCharacter")
-        .unwrap();
+    let mut user_characters = row.column_by_name::<Vec<UserCharacter>>("UserCharacter").unwrap();
     let first_character = user_characters.pop().unwrap();
     assert_eq!(first_character.user_id, *user_id);
     assert_eq!(first_character.character_id, 1);
